class Ball { 
  /*
  Fantastic ball class.
   */
  // Track current position:
  PVector pos;
  // Track current direction & velocity;
  PVector dir;
  // size of the ball
  float bsize;
  // How much to damp the physics.  Higher numbers = less damping.
  float damp = .95;
  // Mass of ball, based on its size.
  float mass;

  Ball() {  
    pos = new PVector(random(-maxSize/2, maxSize/2), random(-maxSize/2, maxSize/2), random(-maxSize/2, maxSize/2));
    dir = new PVector(random(-1, 1), random(-1, 1), random(-1, 1));
    bsize = random(8, maxSize/16);
    mass = PI * pow(bsize, 3);
    //bsize = 64;
  } 

  void update() {
    // Compute the current position.  collide() should be called before this.

    // If ball is outside max size, the bring it back.
    float distToCenter = dist(pos.x, pos.y, pos.z, 0, 0, 0); 
    if (distToCenter > maxSize/2) {
      // The further away the ball is, the greater the force on it to bring it back:
      float percent = percentBetween(distToCenter, maxSize/2, maxSize*2);
      float force = percent * gGravity;

      PVector vecToCenter = PVector.sub(gCenter, pos);
      vecToCenter.normalize();
      vecToCenter.mult(force);
      dir.add(vecToCenter);
    }

    dir.mult(damp);
    pos = PVector.add(pos, dir);
  }

  void render() {
    // Draw our ball
    
    // LOD out ball based on distance to camera:
    float[] camPos = cam.getPosition();
    float distToCam = dist(pos.x, pos.y, pos.z, camPos[0], camPos[1], camPos[2]);
    float pct = percentBetween(distToCam, maxSize/4, maxSize*4);
    int detail = int(lerp(gMaxBallRes, gMinBallRes, pct));
    
    noStroke();
    fill(128);
    pushMatrix();
    translate(pos.x, pos.y, pos.z);
    sphereDetail(detail);
    sphere(bsize);
    popMatrix();
  }

  void collideLight(Light light) {
    // Test to see if this ball has collided with a light.
    // get the distance between the ball and the light:
    float d = dist(pos.x, pos.y, pos.z, light.pos.x, light.pos.y, light.pos.z); 
    // Sum the size of the light and ball:
    float sizeAdd = bsize + light.lsize*1.25;
    // if that size is greater than the distance between, collision!
    if (d < sizeAdd) {
      // get the vector from light to ball, which is the force at point of collision:
      PVector lightToBall = PVector.sub(pos, light.pos);
      lightToBall.normalize();
      lightToBall.mult(light.dir.mag());
      dir.add(lightToBall);
    }
  }

  void collideBall(Ball otherBall) {
    // Test to see if this ball has collided with another ball.
    // Get the distance between the ball and the otherBall:
    float d = dist(pos.x, pos.y, pos.z, otherBall.pos.x, otherBall.pos.y, otherBall.pos.z); 
    // Sum the size of the light and ball:
    float sizeAdd = bsize + otherBall.bsize;
    // if that size is greater than the distance between, collision!
    if (d < sizeAdd) {
      // Get the vector from each ball to each other, since they will both influence
      // one another.
      PVector thisToOther = PVector.sub(otherBall.pos, pos);
      PVector otherToThis = PVector.sub(pos, otherBall.pos);
      thisToOther.normalize();
      otherToThis.normalize();

      // Figure out how much each ball influences each other based on mass:
      float thisForceMult = 0;
      float otherForceMult = 0;
      if (otherBall.mass <= mass) {
        otherForceMult = otherBall.mass / mass;
        thisForceMult = 1 - otherForceMult;
      }
      else {
        thisForceMult = mass / otherBall.mass;
        otherForceMult = 1 - thisForceMult;
      }


      thisToOther.mult(dir.mag() * thisForceMult);
      otherToThis.mult(otherBall.dir.mag() * otherForceMult);

      dir.add(otherToThis);
      otherBall.dir.add(thisToOther);
    }
  }
} 

